import 'package:dio/dio.dart';
import 'package:pmu_project/data/dtos/character_dto.dart';
import 'package:pmu_project/data/mappers/character_mapper.dart';
import 'package:pmu_project/domain/models/card.dart';
import 'package:pmu_project/repositories/api_interface.dart';
import 'package:pretty_dio_logger/pretty_dio_logger.dart';

import '../domain/models/home.dart';

class NarutoRepository extends ApiInterface {
  static final Dio _dio = Dio()
    ..interceptors.add(PrettyDioLogger(
      requestHeader: true,
      requestBody: true,
    ));

  static const String _baseUrl = 'https://dattebayo-api.onrender.com';

  @override
  Future<HomeData?> loadData({
    OnErrorCallback? onError,
    String? q,
    int page = 1,
    int pageSize = 8,
  }) async {
    const String url = '$_baseUrl/characters';
    try {
      final Response<dynamic> response = await _dio.get<Map<dynamic, dynamic>>(
        url,
        queryParameters: {
          'name': q,
          'page': page,
          'limit': pageSize,
        },
      );
      final CharactersDto dto =
          CharactersDto.fromJson(response.data as Map<String, dynamic>);
      final HomeData data = dto.toDomain();
      return data;
    } on DioException catch (e) {
      onError?.call(e.error?.toString());
      return null;
    }
  }
}
